--create a test table and some data
--drop table TEST_TBL
CREATE TABLE TEST_TBL (COL1 INT, COL2 VARCHAR(10), COL3 INT)
GO
INSERT INTO TEST_TBL VALUES (1, 'A', 10)
INSERT INTO TEST_TBL VALUES (1, 'A', 20)
INSERT INTO TEST_TBL VALUES (1, 'A', 30)
INSERT INTO TEST_TBL VALUES (1, 'B', 90)
INSERT INTO TEST_TBL VALUES (2, 'A', 30)
INSERT INTO TEST_TBL VALUES (2, 'A', 100)
INSERT INTO TEST_TBL VALUES (3, 'C', 110)
INSERT INTO TEST_TBL VALUES (3, 'C', 120)
GO

--check the data
SELECT * FROM TEST_TBL
GO

--simple group by
SELECT COL1, COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL, AVG(COL3) AVG_VAL, MAX(COL3) MAX_VAL, MIN(COL3) MIN_VAL
FROM TEST_TBL
GROUP BY COL1, COL2
GO


--add a rollup to the group by
SELECT COL1, COL2, SUM(COL3) AS TOTAL_VAL
FROM TEST_TBL
GROUP BY ROLLUP (COL1, COL2)
GO

--change the order to see the difference
SELECT COL1, COL2, SUM(COL3) AS TOTAL_VAL
FROM TEST_TBL
GROUP BY ROLLUP (COL2, COL1)
GO


-- add a cube (uses all possible permutations of COL1 and COL2)
SELECT COL1, COL2, SUM(COL3) AS TOTAL_VAL
FROM TEST_TBL
GROUP BY CUBE (COL1, COL2)
GO

-- use of grouping function to distinguish null values
SELECT COL1, COL2, SUM(COL3) AS TOTAL_VAL, GROUPING(COL2) AS GRP_VALUE
FROM TEST_TBL
GROUP BY ROLLUP (COL1, COL2)
GO
-- GRP_VALUE = 1 indicates rows generated by ROLLUP

--grouping level to generate the level of grouping
SELECT COL1, COL2, SUM(COL3) AS TOTAL_VAL, GROUPING(COL1) AS C1, GROUPING(COL2) AS C2, GROUPING_ID(COL1, COL2) AS GRP_ID_VALUE
FROM TEST_TBL
GROUP BY ROLLUP (COL1, COL2)
GO

-- how to achieve grouping sets (2005 and before)
SELECT COL1, COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL FROM TEST_TBL GROUP BY COL1, COL2
UNION ALL
SELECT NULL AS COL1, COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL FROM TEST_TBL GROUP BY COL2
UNION ALL
SELECT COL1, NULL AS COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL FROM TEST_TBL GROUP BY COL1
UNION ALL
SELECT NULL AS COL1, NULL AS COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL FROM TEST_TBL
GO


-- grouping sets
SELECT COL1, COL2, COUNT(*) AS CNT_RECORD, SUM(COL3) TOTAL_VAL
FROM TEST_TBL
GROUP BY GROUPING SETS
(
(COL1, COL2),
(COL2),
(COL1),
()
)
GO
